/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.lidar.amapvox.visualization.util.View;
import java.awt.BorderLayout;
import java.awt.Frame;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;

/**
 * Main view of the project.
 */
public class MainView extends View {

    private VisualizationScene scene;
    
    private ToolbarView toolbarView;
    private PropertiesView propertiesView;
    private OptionsView optionsView;
    private ViewView viewView;

    /**
     * Creates new form MainFrame
     * @throws java.lang.Exception
     */
    public MainView() throws Exception {
        super();
    }

    public final void setScene(final VisualizationScene scene) {
        this.scene = scene;
        
        if (optionsView != null) {
            optionsView.getController().setScene(scene);
            optionsView.updateInitialProperties();
        }
        if (viewView != null) {
            viewView.getController().setScene(scene);
        }
        if (toolbarView != null) {
            toolbarView.getController().setScene(scene);
            toolbarView.updateInitialProperties();
        }
        if (propertiesView != null) {
            propertiesView.getController().setScene(scene);
            propertiesView.updatePropertyNames();
        }

        scene.setChangeFileCallback((file) -> {
            propertiesView.updatePropertyNames();
            
            final int keepFilters = JOptionPane.showConfirmDialog(parentView,
                    bundle.getString("confirmDialog_keepFilters"),
                    "",
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE);
            
            if (keepFilters == JOptionPane.NO_OPTION) {
                try {
                    propertiesView.clearFilters();
                } catch (Exception ex) {
                }
            }
        });
    }

    @Override
    protected void initContainer() throws Exception {
        setSize(800, 600);
        setExtendedState(Frame.MAXIMIZED_BOTH);
        setLocationRelativeTo(getParent());
        setLayout(new BorderLayout());
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setIconImage(loadIcon("/icons/visualization.png").getImage());

        addLocaleCallback((locale) -> {
            setTitle(bundle.getString("mainView_Title"));

            if (toolbarView != null) {
                toolbarView.setLocale(locale);
            }
            if (propertiesView != null) {
                propertiesView.setLocale(locale);
            }
            if (optionsView != null) {
                optionsView.setLocale(locale);
            }
            if (viewView != null) {
                viewView.setLocale(locale);
            }
        });
    }

    @Override
    protected void initComponents() throws Exception {
        /* Init the toolbar component */
        toolbarView = new ToolbarView();
        toolbarView.setParentView(this);
        getContentPane().add(toolbarView.getContentPane(), BorderLayout.NORTH);

        /* Init the properties component */
        propertiesView = new PropertiesView();
        propertiesView.setParentView(this);
        propertiesView.setOnChangePropertyCallback((name) -> {
            if (scene != null) {
                final float min = scene.getVoxelSpaceComponent().getFile().getMinPropertyValue(name);
                final float max = scene.getVoxelSpaceComponent().getFile().getMaxPropertyValue(name);
                final float medium = (max - min) / 2;
                viewView.setGradientTexts(Float.toString(min), 
                        Float.toString(medium), Float.toString(max));
            }
        });
        getContentPane().add(propertiesView.getContentPane(), BorderLayout.WEST);

        /* Init the options component */
        optionsView = new OptionsView();
        optionsView.setParentView(this);
        optionsView.setOnChangeColorBarCallback(() -> {
            viewView.updateGradient();
        });
        getContentPane().add(optionsView.getContentPane(), BorderLayout.EAST);

        /* Init the view component */
        viewView = new ViewView();
        viewView.setParentView(this);
        getContentPane().add(viewView.getContentPane(), BorderLayout.CENTER);
    }

}
