/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.commons.util.debug.DebugLevel;
import fr.amap.commons.util.debug.Debugger;
import fr.amap.commons.util.swing.jprogress.JProgressFrame;
import fr.amap.lidar.amapvox.visualization.util.View;
import java.awt.FlowLayout;
import java.io.File;
import java.io.FileNotFoundException;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.jdesktop.swingx.JXButton;
import org.jdesktop.swingx.JXLabel;
import fr.amap.lidar.amapvox.visualization.controllers.ToolbarController;
import fr.amap.lidar.amapvox.visualization.opengl.entities.util.ViewType;
import java.awt.Dimension;

/**
 * toolbar of the main view.
*/
public class ToolbarView extends View {

    private ToolbarController controller;

    private JXButton openButton;
    
    private JXLabel filePathLabel;
    
    private JXButton positiveXViewButton;
    private JXButton negativeXViewButton;
    private JXButton positiveYViewButton;
    private JXButton negativeYViewButton;

    /**
     * Creates new form ToolbarFrame
     * @throws java.lang.Exception
     */
    public ToolbarView() throws Exception {
        super();
        this.controller = new ToolbarController();
    }
    
    public final void updateInitialProperties() {
        filePathLabel.setText(controller.getScene().getVoxelSpaceComponent().getFile().getPath());
        
        controller.getScene().setChangeFileCallback((file) -> {
            filePathLabel.setText(file.getPath());
        });
    }

    public final ToolbarController getController() {
        return controller;
    }

    @Override
    protected void initContainer() throws Exception {
        setLocationRelativeTo(getParent());
        setLayout(new FlowLayout(FlowLayout.LEFT));
    }

    @Override
    protected void initComponents() throws Exception {
        /* Init open button */
        openButton = new JXButton();
        openButton.setIcon(loadIcon("/icons/open.png"));
        addLocaleCallback((locale) -> {
            openButton.setText(bundle.getString("toolbarView_openButton"));
            openButton.setToolTipText(bundle.getString("toolbarView_openButton_tooltip"));
        });
        
        openButton.addActionListener((e) -> {
            final JFileChooser chooser = new JFileChooser();
            chooser.setFileFilter(new FileNameExtensionFilter(
                    bundle.getString("voxelSpaceFile_Name"), "vox"));
            chooser.setAcceptAllFileFilterUsed(false);

            int returnValue = chooser.showOpenDialog(parentView);

            if (returnValue == JFileChooser.APPROVE_OPTION) {
                final File selectedFile = chooser.getSelectedFile();

                if (controller != null) {
                    try {
                        final JProgressFrame progressFrame = new JProgressFrame(parentView);
                        progressFrame.setProgression(0, 1, "Import the voxel space file...");
                        progressFrame.setVisible(true);
                        controller.openVoxelSpaceFile(selectedFile, progressFrame);
                        progressFrame.dispose();
                    } catch (FileNotFoundException ex) {
                        Debugger.debugInterface(ex, DebugLevel.ERROR, parentView);
                    }
                }
            }
        });

        add(openButton);
        
        /* Init file path label */
        filePathLabel = new JXLabel();
        filePathLabel.setText("");
        add(filePathLabel);
        
        /* Init view buttons */
        positiveYViewButton = new JXButton();
        positiveYViewButton.setPreferredSize(new Dimension(45, 45));
        positiveYViewButton.setText(null);
        positiveYViewButton.setIcon(loadIcon("/icons/viewYpos.png"));
        positiveYViewButton.addActionListener((e) -> {
            controller.setCameraView(ViewType.Y_POS);
        });
        addLocaleCallback((locale) -> {
            positiveYViewButton.setToolTipText(bundle.getString("toolbarView_positiveYViewButton_tooltip"));
        });
        add(positiveYViewButton);
        
        negativeYViewButton = new JXButton();
        negativeYViewButton.setPreferredSize(new Dimension(45, 45));
        negativeYViewButton.setText(null);
        negativeYViewButton.setIcon(loadIcon("/icons/viewYneg.png"));
        negativeYViewButton.addActionListener((e) -> {
            controller.setCameraView(ViewType.Y_NEG);
        });
        addLocaleCallback((locale) -> {
            negativeYViewButton.setToolTipText(bundle.getString("toolbarView_negativeYViewButton_tooltip"));
        });
        add(negativeYViewButton);
        
        positiveXViewButton = new JXButton();
        positiveXViewButton.setPreferredSize(new Dimension(45, 45));
        positiveXViewButton.setText(null);
        positiveXViewButton.setIcon(loadIcon("/icons/viewXpos.png"));
        positiveXViewButton.addActionListener((e) -> {
            controller.setCameraView(ViewType.X_POS);
        });
        addLocaleCallback((locale) -> {
            positiveXViewButton.setToolTipText(bundle.getString("toolbarView_positiveXViewButton_tooltip"));
        });
        add(positiveXViewButton);
        
        negativeXViewButton = new JXButton();
        negativeXViewButton.setPreferredSize(new Dimension(45, 45));
        negativeXViewButton.setText(null);
        negativeXViewButton.setIcon(loadIcon("/icons/viewXneg.png"));
        negativeXViewButton.addActionListener((e) -> {
            controller.setCameraView(ViewType.X_NEG);
        });
        addLocaleCallback((locale) -> {
            negativeXViewButton.setToolTipText(bundle.getString("toolbarView_negativeXViewButton_tooltip"));
        });
        add(negativeXViewButton);
    }

}
